@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use Mail::Milter::Authentication::Pragmas;
 # ABSTRACT: A Perl Mail Authentication Milter
-our $VERSION = '3.20210113'; # VERSION
+our $VERSION = '3.20210308'; # VERSION
 use Mail::Milter::Authentication::Handler;
 use Mail::Milter::Authentication::Metric;
 use Mail::Milter::Authentication::Protocol::Milter;
@@ -133,6 +133,7 @@ sub pre_loop_hook($self,@) {
     $self->{'metric'}->register_metrics( {
         'forked_children_total' => 'Total number of child processes forked',
         'reaped_children_total' => 'Total number of child processes reaped',
+        'dequeue_files_total' => { help => 'The number of dequeue files queued', type => 'gauge' },
     } );
 
     $self->{'metric'}->register_metrics( Mail::Milter::Authentication::Handler->register_metrics() );
@@ -844,7 +845,7 @@ sub start($args) {
         die;
     }
 
-    $srvargs{'max_dequeue'} = 1;
+    $srvargs{'max_dequeue'} = $config->{'max_dequeue'} // 5;
     $srvargs{'check_for_dequeue'} = $config->{'check_for_dequeue'} // 60;
 
     _warn "==========";
@@ -1157,7 +1158,7 @@ Mail::Milter::Authentication - A Perl Mail Authentication Milter
 
 =head1 VERSION
 
-version 3.20210113
+version 3.20210308
 
 =head1 SYNOPSIS
 
