@@ -4,15 +4,17 @@ use strict;
 use warnings;
 use Mail::Milter::Authentication::Pragmas;
 # ABSTRACT: Handler superclass
-our $VERSION = '3.20210113'; # VERSION
+our $VERSION = '3.20210308'; # VERSION
 use Mail::Milter::Authentication::Exception;
 use Mail::Milter::Authentication::Resolver;
 use Date::Format qw{ time2str };
 use Digest::MD5 qw{ md5_hex };
+use Lock::File;
 use MIME::Base64;
 use Mail::SPF;
 use Net::DNS::Resolver;
 use Net::IP;
+use Proc::ProcessTable;
 use Sereal qw{encode_sereal decode_sereal};
 use Sys::Hostname;
 use Time::HiRes qw{ ualarm gettimeofday };
@@ -1445,13 +1447,70 @@ sub _dequeue_dir($self) {
 
 
 sub get_dequeue_list($self,$key) {
-    my @dequeue_list;
     my $dir = $self->_dequeue_dir;
+    my $dequeue_index_file = $dir.'/dequeue.index';
+    my $dequeue_lock_file = $dir.'/dequeue.lock';
+
+    my $lock = Lock::File->new( $dequeue_lock_file, {} );
+    my $count_new = 0;
+    my $count_allocated = 0;
+    my $count_stale = 0;
+
+    my $dequeue_index = {};
+    my $j = JSON->new->pretty->canonical->utf8;
+
+    # Build a list of Process IDs
+    my $process_ids = {};
+    my $process_table = Proc::ProcessTable->new();
+    foreach my $process ( @{$process_table->table} ) {
+        $process_ids->{$process->pid} = 1;
+    }
+
+    # Read the last state from the index file
+    if ( -e $dequeue_index_file ) {
+        eval {
+            my $body = scalar read_file($dequeue_index_file);
+            $dequeue_index = $j->decode($body);
+        };
+    }
+
+    my @dequeue_list;
     opendir(my $dh, $dir) || die "Failed to open dequeue directory: $!";
+    FILE:
     while (my $file = readdir $dh) {
-        push @dequeue_list, $file if $file =~ /^$key\..*\.dequeue$/;
+        if ( $file =~ /^$key\..*\.dequeue$/ ) {
+            if ( exists ( $dequeue_index->{ $file } ) ) {
+                if ( exists $process_ids->{ $dequeue_index->{$file}->{pid} } ) {
+                    # File exists in the index, and is associated with a currently valid PID
+                    $count_allocated++;
+                    next FILE;
+                }
+                else {
+                    $count_stale++;
+                }
+            }
+            $dequeue_index->{$file} = {
+                pid => $PID,
+            };
+            $count_new++;
+            push @dequeue_list, $file;
+        }
     }
     closedir $dh;
+
+    # Remove deleted files from the dequeue index
+    foreach my $id ( sort keys $dequeue_index->%* ) {
+        my $filepath = join('/',$dir,$id);
+        delete $dequeue_index->{$id} unless -e $filepath;
+    }
+    write_file($dequeue_index_file,{atomic=>1},$j->encode($dequeue_index));
+
+    $lock->unlock;
+
+    $self->metric_set( 'dequeue_files_total', { 'key' => $key, 'state' => 'new' }, $count_new - $count_stale );
+    $self->metric_set( 'dequeue_files_total', { 'key' => $key, 'state' => 'allocated' }, $count_allocated );
+    $self->metric_set( 'dequeue_files_total', { 'key' => $key, 'state' => 'stale' }, $count_stale );
+
     return \@dequeue_list;
 }
 
@@ -2164,7 +2223,7 @@ Mail::Milter::Authentication::Handler - Handler superclass
 
 =head1 VERSION
 
-version 3.20210113
+version 3.20210308
 
 =head1 DESCRIPTION
 
